@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | Zend Signal Handling                                                 |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2008-2018 The PHP Group                                     |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
    All other licensing and usage conditions are those of the PHP Group.
 */
 
- /* $Id: zend_signal.c 320281 2011-12-02 12:14:16Z dmitry $ */
+ /* $Id$ */
 
 #define _GNU_SOURCE
 #include <string.h>
@@ -48,11 +48,23 @@
 #ifdef ZTS
 ZEND_API int zend_signal_globals_id;
 #else
-zend_signal_globals_t zend_signal_globals;
+ZEND_API zend_signal_globals_t zend_signal_globals;
+#endif /* not ZTS */
+
+#define SIGNAL_BEGIN_CRITICAL() \
+	sigset_t oldmask; \
+	zend_sigprocmask(SIG_BLOCK, &global_sigmask, &oldmask);
+#define SIGNAL_END_CRITICAL() \
+	zend_sigprocmask(SIG_SETMASK, &oldmask, NULL);
+
+#ifdef ZTS
+# define zend_sigprocmask(signo, set, oldset) tsrm_sigmask((signo), (set), (oldset))
+#else
+# define zend_sigprocmask(signo, set, oldset) sigprocmask((signo), (set), (oldset))
 #endif
 
-static void zend_signal_handler(int signo, siginfo_t *siginfo, void *context TSRMLS_DC);
-static int zend_signal_register(int signo, void (*handler)(int, siginfo_t*, void*) TSRMLS_DC);
+static void zend_signal_handler(int signo, siginfo_t *siginfo, void *context);
+static int zend_signal_register(int signo, void (*handler)(int, siginfo_t*, void*));
 
 #ifdef __CYGWIN__
 #define TIMEOUT_SIG SIGALRM
@@ -74,22 +86,30 @@ void zend_signal_handler_defer(int signo, siginfo_t *siginfo, void *context)
 {
 	int errno_save = errno;
 	zend_signal_queue_t *queue, *qtmp;
-	TSRMLS_FETCH();
+	zend_bool is_handling_safe = 1;
 
-	if (SIGG(active)) {
-		if (SIGG(depth) == 0) { /* try to handle signal */
-			if (SIGG(blocked) != -1) { /* inverse */
-				SIGG(blocked) = -1; /* signal is not blocked */
+#ifdef ZTS
+	ZEND_TSRMLS_CACHE_UPDATE();
+	/* A signal could hit after TSRM shutdown, in this case globals are already freed. */
+	if (NULL == TSRMLS_CACHE || NULL == TSRMG_BULK_STATIC(zend_signal_globals_id, zend_signal_globals_t *)) {
+		is_handling_safe = 0;
+	}
+#endif
+
+	if (EXPECTED(is_handling_safe && SIGG(active))) {
+		if (UNEXPECTED(SIGG(depth) == 0)) { /* try to handle signal */
+			if (UNEXPECTED(SIGG(blocked))) {
+				SIGG(blocked) = 0;
 			}
-			if (SIGG(running) == 0) {
+			if (EXPECTED(SIGG(running) == 0)) {
 				SIGG(running) = 1;
-				zend_signal_handler(signo, siginfo, context TSRMLS_CC);
+				zend_signal_handler(signo, siginfo, context);
 
 				queue = SIGG(phead);
 				SIGG(phead) = NULL;
 
 				while (queue) {
-					zend_signal_handler(queue->zend_signal.signo, queue->zend_signal.siginfo, queue->zend_signal.context TSRMLS_CC);
+					zend_signal_handler(queue->zend_signal.signo, queue->zend_signal.siginfo, queue->zend_signal.context);
 					qtmp = queue->next;
 					queue->next = SIGG(pavail);
 					queue->zend_signal.signo = 0;
@@ -99,7 +119,7 @@ void zend_signal_handler_defer(int signo, siginfo_t *siginfo, void *context)
 				SIGG(running) = 0;
 			}
 		} else { /* delay signal handling */
-			SIGG(blocked) = 0; /* signal is blocked */
+			SIGG(blocked) = 1; /* signal is blocked */
 
 			if ((queue = SIGG(pavail))) { /* if none available it's simply forgotton */
 				SIGG(pavail) = queue->next;
@@ -123,7 +143,7 @@ void zend_signal_handler_defer(int signo, siginfo_t *siginfo, void *context)
 		}
 	} else {
 		/* need to just run handler if we're inactive and getting a signal */
-		zend_signal_handler(signo, siginfo, context TSRMLS_CC);
+		zend_signal_handler(signo, siginfo, context);
 	}
 
 	errno = errno_save;
@@ -131,12 +151,12 @@ void zend_signal_handler_defer(int signo, siginfo_t *siginfo, void *context)
 
 /* {{{ zend_signal_handler_unblock
  * Handle deferred signal from HANDLE_UNBLOCK_ALARMS */
-ZEND_API void zend_signal_handler_unblock(TSRMLS_D)
+ZEND_API void zend_signal_handler_unblock(void)
 {
 	zend_signal_queue_t *queue;
 	zend_signal_t zend_signal;
 
-	if (SIGG(active)) {
+	if (EXPECTED(SIGG(active))) {
 		SIGNAL_BEGIN_CRITICAL(); /* procmask to protect handler_defer as if it were called by the kernel */
 		queue = SIGG(phead);
 		SIGG(phead) = queue->next;
@@ -154,12 +174,19 @@ ZEND_API void zend_signal_handler_unblock(TSRMLS_D)
 /* {{{ zend_signal_handler
  *  Call the previously registered handler for a signal
  */
-static void zend_signal_handler(int signo, siginfo_t *siginfo, void *context TSRMLS_DC)
+static void zend_signal_handler(int signo, siginfo_t *siginfo, void *context)
 {
 	int errno_save = errno;
-	struct sigaction sa = {{0}};
+	struct sigaction sa;
 	sigset_t sigset;
-	zend_signal_entry_t p_sig = SIGG(handlers)[signo-1];
+	zend_signal_entry_t p_sig;
+#ifdef ZTS
+	if (NULL == TSRMLS_CACHE || NULL == TSRMG_BULK_STATIC(zend_signal_globals_id, zend_signal_globals_t *)) {
+		p_sig.flags = 0;
+		p_sig.handler = SIG_DFL;
+	} else
+#endif
+	p_sig = SIGG(handlers)[signo-1];
 
 	if (p_sig.handler == SIG_DFL) { /* raise default handler */
 		if (sigaction(signo, NULL, &sa) == 0) {
@@ -171,11 +198,19 @@ static void zend_signal_handler(int signo, siginfo_t *siginfo, void *context TSR
 
 			if (sigaction(signo, &sa, NULL) == 0) {
 				/* throw away any blocked signals */
-				sigprocmask(SIG_UNBLOCK, &sigset, NULL);
-				raise(signo);
+				zend_sigprocmask(SIG_UNBLOCK, &sigset, NULL);
+#ifdef ZTS
+# define RAISE_ERROR "raise() failed\n"
+				if (raise(signo) != 0) {
+					/* On some systems raise() fails with errno 3: No such process */
+					kill(getpid(), signo);
+				}
+#else
+				kill(getpid(), signo);
+#endif
 			}
 		}
-	} else if (p_sig.handler != SIG_IGN) { /* ignore SIG_IGN */
+	} else if (p_sig.handler != SIG_IGN) {
 		if (p_sig.flags & SA_SIGINFO) {
 			if (p_sig.flags & SA_RESETHAND) {
 				SIGG(handlers)[signo-1].flags   = 0;
@@ -192,9 +227,9 @@ static void zend_signal_handler(int signo, siginfo_t *siginfo, void *context TSR
 
 /* {{{ zend_sigaction
  *  Register a signal handler that will be deferred in critical sections */
-ZEND_API int zend_sigaction(int signo, const struct sigaction *act, struct sigaction *oldact TSRMLS_DC)
+ZEND_API int zend_sigaction(int signo, const struct sigaction *act, struct sigaction *oldact)
 {
-	struct sigaction sa = {{0}};
+	struct sigaction sa;
 	sigset_t sigset;
 
 	if (oldact != NULL) {
@@ -210,12 +245,17 @@ ZEND_API int zend_sigaction(int signo, const struct sigaction *act, struct sigac
 			SIGG(handlers)[signo-1].handler = (void *) act->sa_handler;
 		}
 
-		sa.sa_flags     = SA_SIGINFO | (act->sa_flags & SA_FLAGS_MASK);
-		sa.sa_sigaction = zend_signal_handler_defer;
-		sa.sa_mask      = global_sigmask;
+		memset(&sa, 0, sizeof(sa));
+		if (SIGG(handlers)[signo-1].handler == (void *) SIG_IGN) {
+			sa.sa_sigaction = (void *) SIG_IGN;
+		} else {
+			sa.sa_flags     = SA_SIGINFO | (act->sa_flags & SA_FLAGS_MASK);
+			sa.sa_sigaction = zend_signal_handler_defer;
+			sa.sa_mask      = global_sigmask;
+		}
 
 		if (sigaction(signo, &sa, NULL) < 0) {
-			zend_error(E_WARNING, "Error installing signal handler for %d", signo);
+			zend_error_noreturn(E_ERROR, "Error installing signal handler for %d", signo);
 		}
 
 		/* unsure this signal is not blocked */
@@ -230,15 +270,16 @@ ZEND_API int zend_sigaction(int signo, const struct sigaction *act, struct sigac
 
 /* {{{ zend_signal
  *  Register a signal handler that will be deferred in critical sections */
-ZEND_API int zend_signal(int signo, void (*handler)(int) TSRMLS_DC)
+ZEND_API int zend_signal(int signo, void (*handler)(int))
 {
-	struct sigaction sa = {{0}};
+	struct sigaction sa;
 
+	memset(&sa, 0, sizeof(sa));
 	sa.sa_flags   = 0;
 	sa.sa_handler = handler;
 	sa.sa_mask    = global_sigmask;
 
-	return zend_sigaction(signo, &sa, NULL TSRMLS_CC);
+	return zend_sigaction(signo, &sa, NULL);
 }
 /* }}} */
 
@@ -246,9 +287,9 @@ ZEND_API int zend_signal(int signo, void (*handler)(int) TSRMLS_DC)
  *  Set a handler for a signal we want to defer.
  *  Previously set handler must have been saved before.
  */
-static int zend_signal_register(int signo, void (*handler)(int, siginfo_t*, void*) TSRMLS_DC)
+static int zend_signal_register(int signo, void (*handler)(int, siginfo_t*, void*))
 {
-	struct sigaction sa = {{0}};
+	struct sigaction sa;
 
 	if (sigaction(signo, NULL, &sa) == 0) {
 		if ((sa.sa_flags & SA_SIGINFO) && sa.sa_sigaction == handler) {
@@ -267,7 +308,7 @@ static int zend_signal_register(int signo, void (*handler)(int, siginfo_t*, void
 		sa.sa_mask      = global_sigmask;
 
 		if (sigaction(signo, &sa, NULL) < 0) {
-			zend_error(E_WARNING, "Error installing signal handler for %d", signo);
+			zend_error_noreturn(E_ERROR, "Error installing signal handler for %d", signo);
 		}
 
 		return SUCCESS;
@@ -277,14 +318,14 @@ static int zend_signal_register(int signo, void (*handler)(int, siginfo_t*, void
 
 /* {{{ zend_signal_activate
  *  Install our signal handlers, per request */
-void zend_signal_activate(TSRMLS_D)
+void zend_signal_activate(void)
 {
-	int x;
+	size_t x;
 
 	memcpy(&SIGG(handlers), &global_orig_handlers, sizeof(global_orig_handlers));
 
-	for (x=0; x < sizeof(zend_sigs) / sizeof(*zend_sigs); x++) {
-		zend_signal_register(zend_sigs[x], zend_signal_handler_defer TSRMLS_CC);
+	for (x = 0; x < sizeof(zend_sigs) / sizeof(*zend_sigs); x++) {
+		zend_signal_register(zend_sigs[x], zend_signal_handler_defer);
 	}
 
 	SIGG(active) = 1;
@@ -293,17 +334,18 @@ void zend_signal_activate(TSRMLS_D)
 
 /* {{{ zend_signal_deactivate
  * */
-void zend_signal_deactivate(TSRMLS_D)
+void zend_signal_deactivate(void)
 {
-	int x;
-	struct sigaction sa = {{0}};
 
 	if (SIGG(check)) {
+		size_t x;
+		struct sigaction sa;
+
 		if (SIGG(depth) != 0) {
 			zend_error(E_CORE_WARNING, "zend_signal: shutdown with non-zero blocking depth (%d)", SIGG(depth));
 		}
 		/* did anyone steal our installed handler */
-		for (x=0; x < sizeof(zend_sigs) / sizeof(*zend_sigs); x++) {
+		for (x = 0; x < sizeof(zend_sigs) / sizeof(*zend_sigs); x++) {
 			sigaction(zend_sigs[x], NULL, &sa);
 			if (sa.sa_sigaction != zend_signal_handler_defer) {
 				zend_error(E_CORE_WARNING, "zend_signal: handler was replaced for signal (%d) after startup", zend_sigs[x]);
@@ -314,18 +356,17 @@ void zend_signal_deactivate(TSRMLS_D)
 	SIGNAL_BEGIN_CRITICAL();
 	SIGG(active) = 0;
 	SIGG(running) = 0;
-	SIGG(blocked) = -1;
+	SIGG(blocked) = 0;
 	SIGG(depth) = 0;
 	SIGNAL_END_CRITICAL();
 }
 /* }}} */
 
-static void zend_signal_globals_ctor(zend_signal_globals_t *zend_signal_globals TSRMLS_DC)
+static void zend_signal_globals_ctor(zend_signal_globals_t *zend_signal_globals) /* {{{ */
 {
 	size_t x;
 
 	memset(zend_signal_globals, 0, sizeof(*zend_signal_globals));
-	zend_signal_globals->blocked = -1;
 
 	for (x = 0; x < sizeof(zend_signal_globals->pstorage) / sizeof(*zend_signal_globals->pstorage); ++x) {
 		zend_signal_queue_t *queue = &zend_signal_globals->pstorage[x];
@@ -334,21 +375,35 @@ static void zend_signal_globals_ctor(zend_signal_globals_t *zend_signal_globals
 		zend_signal_globals->pavail = queue;
 	}
 }
+/* }}} */
 
-static void zend_signal_globals_dtor(zend_signal_globals_t *zend_signal_globals TSRMLS_DC)
+void zend_signal_init(void) /* {{{ */
 {
-	zend_signal_globals->blocked = -1;
+	int signo;
+	struct sigaction sa;
+
+	/* Save previously registered signal handlers into orig_handlers */
+	memset(&global_orig_handlers, 0, sizeof(global_orig_handlers));
+	for (signo = 1; signo < NSIG; ++signo) {
+		if (sigaction(signo, NULL, &sa) == 0) {
+			global_orig_handlers[signo-1].flags = sa.sa_flags;
+			if (sa.sa_flags & SA_SIGINFO) {
+				global_orig_handlers[signo-1].handler = (void *) sa.sa_sigaction;
+			} else {
+				global_orig_handlers[signo-1].handler = (void *) sa.sa_handler;
+			}
+		}
+	}
 }
+/* }}} */
 
 /* {{{ zend_signal_startup
  * alloc zend signal globals */
-void zend_signal_startup()
+void zend_signal_startup(void)
 {
-	int signo;
-	struct sigaction sa = {{0}};
 
 #ifdef ZTS
-	ts_allocate_id(&zend_signal_globals_id, sizeof(zend_signal_globals_t), (ts_allocate_ctor) zend_signal_globals_ctor, (ts_allocate_dtor) zend_signal_globals_dtor);
+	ts_allocate_id(&zend_signal_globals_id, sizeof(zend_signal_globals_t), (ts_allocate_ctor) zend_signal_globals_ctor, NULL);
 #else
 	zend_signal_globals_ctor(&zend_signal_globals);
 #endif
@@ -375,28 +430,7 @@ void zend_signal_startup()
 	sigdelset(&global_sigmask, SIGTRAP);
 #endif
 
-	/* Save previously registered signal handlers into orig_handlers */
-	memset(&global_orig_handlers, 0, sizeof(global_orig_handlers));
-	for (signo = 1; signo < NSIG; ++signo) {
-		if (sigaction(signo, NULL, &sa) == 0) {
-			global_orig_handlers[signo-1].flags = sa.sa_flags;
-			if (sa.sa_flags & SA_SIGINFO) {
-				global_orig_handlers[signo-1].handler = (void *) sa.sa_sigaction;
-			} else {
-				global_orig_handlers[signo-1].handler = (void *) sa.sa_handler;
-			}
-		}
-	}
-}
-/* }}} */
-
-/* {{{ zend_signal_shutdown
- * called by zend_shutdown */
-void zend_signal_shutdown(TSRMLS_D)
-{
-#ifndef ZTS
-	zend_signal_globals_dtor(&zend_signal_globals);
-#endif
+	zend_signal_init();
 }
 /* }}} */
 
