@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,8 +16,8 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_mbregex.h 321634 2012-01-01 13:15:04Z felipe $ */
- 
+/* $Id$ */
+
 #ifndef _PHP_MBREGEX_H
 #define _PHP_MBREGEX_H
 
@@ -34,6 +34,7 @@
 	PHP_FE(mb_eregi,			arginfo_mb_eregi) \
 	PHP_FE(mb_ereg_replace,			arginfo_mb_ereg_replace) \
 	PHP_FE(mb_eregi_replace,			arginfo_mb_eregi_replace) \
+	PHP_FE(mb_ereg_replace_callback,			arginfo_mb_ereg_replace_callback) \
 	PHP_FE(mb_split,					arginfo_mb_split) \
 	PHP_FE(mb_ereg_match,			arginfo_mb_ereg_match) \
 	PHP_FE(mb_ereg_search,			arginfo_mb_ereg_search) \
@@ -69,18 +70,19 @@ PHP_MINFO_FUNCTION(mb_regex);
 
 typedef struct _zend_mb_regex_globals zend_mb_regex_globals;
 
-zend_mb_regex_globals *php_mb_regex_globals_alloc(TSRMLS_D);
-void php_mb_regex_globals_free(zend_mb_regex_globals *pglobals TSRMLS_DC);
-int php_mb_regex_set_mbctype(const char *enc TSRMLS_DC);
-int php_mb_regex_set_default_mbctype(const char *encname TSRMLS_DC);
-const char *php_mb_regex_get_mbctype(TSRMLS_D);
-const char *php_mb_regex_get_default_mbctype(TSRMLS_D);
+zend_mb_regex_globals *php_mb_regex_globals_alloc(void);
+void php_mb_regex_globals_free(zend_mb_regex_globals *pglobals);
+int php_mb_regex_set_mbctype(const char *enc);
+int php_mb_regex_set_default_mbctype(const char *encname);
+const char *php_mb_regex_get_mbctype(void);
+const char *php_mb_regex_get_default_mbctype(void);
 
 PHP_FUNCTION(mb_regex_encoding);
 PHP_FUNCTION(mb_ereg);
 PHP_FUNCTION(mb_eregi);
 PHP_FUNCTION(mb_ereg_replace);
 PHP_FUNCTION(mb_eregi_replace);
+PHP_FUNCTION(mb_ereg_replace_callback);
 PHP_FUNCTION(mb_split);
 PHP_FUNCTION(mb_ereg_match);
 PHP_FUNCTION(mb_ereg_search);
@@ -104,4 +106,3 @@ PHP_FUNCTION(mb_regex_set_options);
  * vim600: noet sw=4 ts=4 fdm=marker
  * vim<600: noet sw=4 ts=4
  */
-
