@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | phar php single-file executable PHP extension                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2005-2016 The PHP Group                                |
+  | Copyright (c) 2005-2018 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,26 +26,26 @@
 PHAR_FUNC(phar_opendir) /* {{{ */
 {
 	char *filename;
-	int filename_len;
+	size_t filename_len;
 	zval *zcontext = NULL;
 
 	if (!PHAR_G(intercepted)) {
 		goto skip_phar;
 	}
 
-	if ((PHAR_GLOBALS->phar_fname_map.arBuckets && !zend_hash_num_elements(&(PHAR_GLOBALS->phar_fname_map)))
-		&& !cached_phars.arBuckets) {
+	if ((PHAR_G(phar_fname_map.u.flags) && !zend_hash_num_elements(&(PHAR_G(phar_fname_map))))
+		&& !cached_phars.u.flags) {
 		goto skip_phar;
 	}
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p|z", &filename, &filename_len, &zcontext) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p|z", &filename, &filename_len, &zcontext) == FAILURE) {
 		return;
 	}
 
 	if (!IS_ABSOLUTE_PATH(filename, filename_len) && !strstr(filename, "://")) {
 		char *arch, *entry, *fname;
 		int arch_len, entry_len, fname_len;
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		/* we are checking for existence of a file within the relative path.  Chances are good that this is
 		   retrieving something from within the phar archive */
@@ -54,7 +54,7 @@ PHAR_FUNC(phar_opendir) /* {{{ */
 			goto skip_phar;
 		}
 		fname_len = strlen(fname);
-		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 			php_stream_context *context = NULL;
 			php_stream *stream;
 			char *name;
@@ -64,7 +64,7 @@ PHAR_FUNC(phar_opendir) /* {{{ */
 			/* fopen within phar, if :// is not in the url, then prepend phar://<archive>/ */
 			entry_len = filename_len;
 			/* retrieving a file within the current directory, so use this if possible */
-			entry = phar_fix_filepath(entry, &entry_len, 1 TSRMLS_CC);
+			entry = phar_fix_filepath(entry, &entry_len, 1);
 
 			if (entry[0] == '/') {
 				spprintf(&name, 4096, "phar://%s%s", arch, entry);
@@ -94,41 +94,41 @@ skip_phar:
 PHAR_FUNC(phar_file_get_contents) /* {{{ */
 {
 	char *filename;
-	int filename_len;
-	char *contents;
+	size_t filename_len;
+	zend_string *contents;
 	zend_bool use_include_path = 0;
 	php_stream *stream;
-	int len;
-	long offset = -1;
-	long maxlen = PHP_STREAM_COPY_ALL;
+	zend_long offset = -1;
+	zend_long maxlen = PHP_STREAM_COPY_ALL;
 	zval *zcontext = NULL;
 
 	if (!PHAR_G(intercepted)) {
 		goto skip_phar;
 	}
 
-	if ((PHAR_GLOBALS->phar_fname_map.arBuckets && !zend_hash_num_elements(&(PHAR_GLOBALS->phar_fname_map)))
-		&& !cached_phars.arBuckets) {
+	if ((PHAR_G(phar_fname_map.u.flags) && !zend_hash_num_elements(&(PHAR_G(phar_fname_map))))
+		&& !cached_phars.u.flags) {
 		goto skip_phar;
 	}
 
 	/* Parse arguments */
-	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "p|br!ll", &filename, &filename_len, &use_include_path, &zcontext, &offset, &maxlen) == FAILURE) {
+	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS(), "p|br!ll", &filename, &filename_len, &use_include_path, &zcontext, &offset, &maxlen) == FAILURE) {
 		goto skip_phar;
 	}
 
 	if (use_include_path || (!IS_ABSOLUTE_PATH(filename, filename_len) && !strstr(filename, "://"))) {
 		char *arch, *entry, *fname;
+		zend_string *entry_str = NULL;
 		int arch_len, entry_len, fname_len;
 		php_stream_context *context = NULL;
 
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		if (strncasecmp(fname, "phar://", 7)) {
 			goto skip_phar;
 		}
 		fname_len = strlen(fname);
-		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 			char *name;
 			phar_archive_data *phar;
 
@@ -139,18 +139,18 @@ PHAR_FUNC(phar_file_get_contents) /* {{{ */
 
 			if (ZEND_NUM_ARGS() == 5 && maxlen < 0) {
 				efree(arch);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "length must be greater than or equal to zero");
+				php_error_docref(NULL, E_WARNING, "length must be greater than or equal to zero");
 				RETURN_FALSE;
 			}
 
 			/* retrieving a file defaults to within the current directory, so use this if possible */
-			if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL TSRMLS_CC)) {
+			if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL)) {
 				efree(arch);
 				goto skip_phar;
 			}
 			if (use_include_path) {
-				if ((entry = phar_find_in_include_path(entry, entry_len, NULL TSRMLS_CC))) {
-					name = entry;
+				if ((entry_str = phar_find_in_include_path(entry, entry_len, NULL))) {
+					name = ZSTR_VAL(entry_str);
 					goto phar_it;
 				} else {
 					/* this file is not in the phar, use the original path */
@@ -158,9 +158,9 @@ PHAR_FUNC(phar_file_get_contents) /* {{{ */
 					goto skip_phar;
 				}
 			} else {
-				entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1 TSRMLS_CC);
+				entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1);
 				if (entry[0] == '/') {
-					if (!zend_hash_exists(&(phar->manifest), entry + 1, entry_len - 1)) {
+					if (!zend_hash_str_exists(&(phar->manifest), entry + 1, entry_len - 1)) {
 						/* this file is not in the phar, use the original path */
 notfound:
 						efree(arch);
@@ -168,7 +168,7 @@ notfound:
 						goto skip_phar;
 					}
 				} else {
-					if (!zend_hash_exists(&(phar->manifest), entry, entry_len)) {
+					if (!zend_hash_str_exists(&(phar->manifest), entry, entry_len)) {
 						goto notfound;
 					}
 				}
@@ -189,29 +189,28 @@ phar_it:
 				context = php_stream_context_from_zval(zcontext, 0);
 			}
 			stream = php_stream_open_wrapper_ex(name, "rb", 0 | REPORT_ERRORS, NULL, context);
-			efree(name);
+			if (entry_str) {
+				zend_string_release(entry_str);
+			} else {
+				efree(name);
+			}
 
 			if (!stream) {
 				RETURN_FALSE;
 			}
 
 			if (offset > 0 && php_stream_seek(stream, offset, SEEK_SET) < 0) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to position %ld in the stream", offset);
+				php_error_docref(NULL, E_WARNING, "Failed to seek to position " ZEND_LONG_FMT " in the stream", offset);
 				php_stream_close(stream);
 				RETURN_FALSE;
 			}
 
 			/* uses mmap if possible */
-			if ((len = php_stream_copy_to_mem(stream, &contents, maxlen, 0)) > 0) {
-#if PHP_API_VERSION < 20100412
-				if (PG(magic_quotes_runtime)) {
-					int newlen;
-					contents = php_addslashes(contents, len, &newlen, 1 TSRMLS_CC); /* 1 = free source string */
-					len = newlen;
-				}
-#endif
-				RETVAL_STRINGL(contents, len, 0);
-			} else if (len == 0) {
+			contents = php_stream_copy_to_mem(stream, maxlen, 0);
+			if (contents && ZSTR_LEN(contents) > 0) {
+				RETVAL_STR(contents);
+			} else if (contents) {
+				zend_string_release(contents);
 				RETVAL_EMPTY_STRING();
 			} else {
 				RETVAL_FALSE;
@@ -230,7 +229,7 @@ skip_phar:
 PHAR_FUNC(phar_readfile) /* {{{ */
 {
 	char *filename;
-	int filename_len;
+	size_t filename_len;
 	int size = 0;
 	zend_bool use_include_path = 0;
 	zval *zcontext = NULL;
@@ -240,26 +239,27 @@ PHAR_FUNC(phar_readfile) /* {{{ */
 		goto skip_phar;
 	}
 
-	if ((PHAR_GLOBALS->phar_fname_map.arBuckets && !zend_hash_num_elements(&(PHAR_GLOBALS->phar_fname_map)))
-		&& !cached_phars.arBuckets) {
+	if ((PHAR_G(phar_fname_map.u.flags) && !zend_hash_num_elements(&(PHAR_G(phar_fname_map))))
+		&& !cached_phars.u.flags) {
 		goto skip_phar;
 	}
-	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "p|br!", &filename, &filename_len, &use_include_path, &zcontext) == FAILURE) {
+	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS(), "p|br!", &filename, &filename_len, &use_include_path, &zcontext) == FAILURE) {
 		goto skip_phar;
 	}
 	if (use_include_path || (!IS_ABSOLUTE_PATH(filename, filename_len) && !strstr(filename, "://"))) {
 		char *arch, *entry, *fname;
+		zend_string *entry_str = NULL;
 		int arch_len, entry_len, fname_len;
 		php_stream_context *context = NULL;
 		char *name;
 		phar_archive_data *phar;
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		if (strncasecmp(fname, "phar://", 7)) {
 			goto skip_phar;
 		}
 		fname_len = strlen(fname);
-		if (FAILURE == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (FAILURE == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 			goto skip_phar;
 		}
 
@@ -268,22 +268,22 @@ PHAR_FUNC(phar_readfile) /* {{{ */
 		/* fopen within phar, if :// is not in the url, then prepend phar://<archive>/ */
 		entry_len = filename_len;
 		/* retrieving a file defaults to within the current directory, so use this if possible */
-		if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL TSRMLS_CC)) {
+		if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL)) {
 			efree(arch);
 			goto skip_phar;
 		}
 		if (use_include_path) {
-			if (!(entry = phar_find_in_include_path(entry, entry_len, NULL TSRMLS_CC))) {
+			if (!(entry_str = phar_find_in_include_path(entry, entry_len, NULL))) {
 				/* this file is not in the phar, use the original path */
 				efree(arch);
 				goto skip_phar;
 			} else {
-				name = entry;
+				name = ZSTR_VAL(entry_str);
 			}
 		} else {
-			entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1 TSRMLS_CC);
+			entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1);
 			if (entry[0] == '/') {
-				if (!zend_hash_exists(&(phar->manifest), entry + 1, entry_len - 1)) {
+				if (!zend_hash_str_exists(&(phar->manifest), entry + 1, entry_len - 1)) {
 					/* this file is not in the phar, use the original path */
 notfound:
 					efree(entry);
@@ -291,7 +291,7 @@ notfound:
 					goto skip_phar;
 				}
 			} else {
-				if (!zend_hash_exists(&(phar->manifest), entry, entry_len)) {
+				if (!zend_hash_str_exists(&(phar->manifest), entry, entry_len)) {
 					goto notfound;
 				}
 			}
@@ -307,7 +307,11 @@ notfound:
 		efree(arch);
 		context = php_stream_context_from_zval(zcontext, 0);
 		stream = php_stream_open_wrapper_ex(name, "rb", 0 | REPORT_ERRORS, NULL, context);
-		efree(name);
+		if (entry_str) {
+			zend_string_release(entry_str);
+		} else {
+			efree(name);
+		}
 		if (stream == NULL) {
 			RETURN_FALSE;
 		}
@@ -326,7 +330,7 @@ skip_phar:
 PHAR_FUNC(phar_fopen) /* {{{ */
 {
 	char *filename, *mode;
-	int filename_len, mode_len;
+	size_t filename_len, mode_len;
 	zend_bool use_include_path = 0;
 	zval *zcontext = NULL;
 	php_stream *stream;
@@ -335,27 +339,28 @@ PHAR_FUNC(phar_fopen) /* {{{ */
 		goto skip_phar;
 	}
 
-	if ((PHAR_GLOBALS->phar_fname_map.arBuckets && !zend_hash_num_elements(&(PHAR_GLOBALS->phar_fname_map)))
-		&& !cached_phars.arBuckets) {
+	if ((PHAR_G(phar_fname_map.u.flags) && !zend_hash_num_elements(&(PHAR_G(phar_fname_map))))
+		&& !cached_phars.u.flags) {
 		/* no need to check, include_path not even specified in fopen/ no active phars */
 		goto skip_phar;
 	}
-	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "ps|br", &filename, &filename_len, &mode, &mode_len, &use_include_path, &zcontext) == FAILURE) {
+	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS(), "ps|br", &filename, &filename_len, &mode, &mode_len, &use_include_path, &zcontext) == FAILURE) {
 		goto skip_phar;
 	}
 	if (use_include_path || (!IS_ABSOLUTE_PATH(filename, filename_len) && !strstr(filename, "://"))) {
 		char *arch, *entry, *fname;
+		zend_string *entry_str = NULL;
 		int arch_len, entry_len, fname_len;
 		php_stream_context *context = NULL;
 		char *name;
 		phar_archive_data *phar;
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		if (strncasecmp(fname, "phar://", 7)) {
 			goto skip_phar;
 		}
 		fname_len = strlen(fname);
-		if (FAILURE == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (FAILURE == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 			goto skip_phar;
 		}
 
@@ -364,22 +369,22 @@ PHAR_FUNC(phar_fopen) /* {{{ */
 		/* fopen within phar, if :// is not in the url, then prepend phar://<archive>/ */
 		entry_len = filename_len;
 		/* retrieving a file defaults to within the current directory, so use this if possible */
-		if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL TSRMLS_CC)) {
+		if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL)) {
 			efree(arch);
 			goto skip_phar;
 		}
 		if (use_include_path) {
-			if (!(entry = phar_find_in_include_path(entry, entry_len, NULL TSRMLS_CC))) {
+			if (!(entry_str = phar_find_in_include_path(entry, entry_len, NULL))) {
 				/* this file is not in the phar, use the original path */
 				efree(arch);
 				goto skip_phar;
 			} else {
-				name = entry;
+				name = ZSTR_VAL(entry_str);
 			}
 		} else {
-			entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1 TSRMLS_CC);
+			entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1);
 			if (entry[0] == '/') {
-				if (!zend_hash_exists(&(phar->manifest), entry + 1, entry_len - 1)) {
+				if (!zend_hash_str_exists(&(phar->manifest), entry + 1, entry_len - 1)) {
 					/* this file is not in the phar, use the original path */
 notfound:
 					efree(entry);
@@ -387,7 +392,7 @@ notfound:
 					goto skip_phar;
 				}
 			} else {
-				if (!zend_hash_exists(&(phar->manifest), entry, entry_len)) {
+				if (!zend_hash_str_exists(&(phar->manifest), entry, entry_len)) {
 					/* this file is not in the phar, use the original path */
 					goto notfound;
 				}
@@ -404,13 +409,17 @@ notfound:
 		efree(arch);
 		context = php_stream_context_from_zval(zcontext, 0);
 		stream = php_stream_open_wrapper_ex(name, mode, 0 | REPORT_ERRORS, NULL, context);
-		efree(name);
+		if (entry_str) {
+			zend_string_release(entry_str);
+		} else {
+			efree(name);
+		}
 		if (stream == NULL) {
 			RETURN_FALSE;
 		}
 		php_stream_to_zval(stream, return_value);
 		if (zcontext) {
-			zend_list_addref(Z_RESVAL_P(zcontext));
+			Z_ADDREF_P(zcontext);
 		}
 		return;
 	}
@@ -420,18 +429,6 @@ skip_phar:
 }
 /* }}} */
 
-#ifndef S_ISDIR
-#define S_ISDIR(mode)	(((mode)&S_IFMT) == S_IFDIR)
-#endif
-#ifndef S_ISREG
-#define S_ISREG(mode)	(((mode)&S_IFMT) == S_IFREG)
-#endif
-#ifndef S_ISLNK
-#define S_ISLNK(mode)	(((mode)&S_IFMT) == S_IFLNK)
-#endif
-
-#define S_IXROOT ( S_IXUSR | S_IXGRP | S_IXOTH )
-
 #define IS_LINK_OPERATION(__t) ((__t) == FS_TYPE || (__t) == FS_IS_LINK || (__t) == FS_LSTAT)
 #define IS_EXISTS_CHECK(__t) ((__t) == FS_EXISTS  || (__t) == FS_IS_W || (__t) == FS_IS_R || (__t) == FS_IS_X || (__t) == FS_IS_FILE || (__t) == FS_IS_DIR || (__t) == FS_IS_LINK)
 #define IS_ABLE_CHECK(__t) ((__t) == FS_IS_R || (__t) == FS_IS_W || (__t) == FS_IS_X)
@@ -439,10 +436,10 @@ skip_phar:
 
 /* {{{ php_stat
  */
-static void phar_fancy_stat(struct stat *stat_sb, int type, zval *return_value TSRMLS_DC)
+static void phar_fancy_stat(zend_stat_t *stat_sb, int type, zval *return_value)
 {
-	zval *stat_dev, *stat_ino, *stat_mode, *stat_nlink, *stat_uid, *stat_gid, *stat_rdev,
-		 *stat_size, *stat_atime, *stat_mtime, *stat_ctime, *stat_blksize, *stat_blocks;
+	zval stat_dev, stat_ino, stat_mode, stat_nlink, stat_uid, stat_gid, stat_rdev,
+		 stat_size, stat_atime, stat_mtime, stat_ctime, stat_blksize, stat_blocks;
 	int rmask=S_IROTH, wmask=S_IWOTH, xmask=S_IXOTH; /* access rights defaults to other */
 	char *stat_sb_names[13] = {
 		"dev", "ino", "mode", "nlink", "uid", "gid", "rdev",
@@ -483,43 +480,43 @@ static void phar_fancy_stat(struct stat *stat_sb, int type, zval *return_value T
 
 	switch (type) {
 	case FS_PERMS:
-		RETURN_LONG((long)stat_sb->st_mode);
+		RETURN_LONG((zend_long)stat_sb->st_mode);
 	case FS_INODE:
-		RETURN_LONG((long)stat_sb->st_ino);
+		RETURN_LONG((zend_long)stat_sb->st_ino);
 	case FS_SIZE:
-		RETURN_LONG((long)stat_sb->st_size);
+		RETURN_LONG((zend_long)stat_sb->st_size);
 	case FS_OWNER:
-		RETURN_LONG((long)stat_sb->st_uid);
+		RETURN_LONG((zend_long)stat_sb->st_uid);
 	case FS_GROUP:
-		RETURN_LONG((long)stat_sb->st_gid);
+		RETURN_LONG((zend_long)stat_sb->st_gid);
 	case FS_ATIME:
 #ifdef NETWARE
-		RETURN_LONG((long)stat_sb->st_atime.tv_sec);
+		RETURN_LONG((zend_long)stat_sb->st_atime.tv_sec);
 #else
-		RETURN_LONG((long)stat_sb->st_atime);
+		RETURN_LONG((zend_long)stat_sb->st_atime);
 #endif
 	case FS_MTIME:
 #ifdef NETWARE
-		RETURN_LONG((long)stat_sb->st_mtime.tv_sec);
+		RETURN_LONG((zend_long)stat_sb->st_mtime.tv_sec);
 #else
-		RETURN_LONG((long)stat_sb->st_mtime);
+		RETURN_LONG((zend_long)stat_sb->st_mtime);
 #endif
 	case FS_CTIME:
 #ifdef NETWARE
-		RETURN_LONG((long)stat_sb->st_ctime.tv_sec);
+		RETURN_LONG((zend_long)stat_sb->st_ctime.tv_sec);
 #else
-		RETURN_LONG((long)stat_sb->st_ctime);
+		RETURN_LONG((zend_long)stat_sb->st_ctime);
 #endif
 	case FS_TYPE:
 		if (S_ISLNK(stat_sb->st_mode)) {
-			RETURN_STRING("link", 1);
+			RETURN_STRING("link");
 		}
 		switch(stat_sb->st_mode & S_IFMT) {
-		case S_IFDIR: RETURN_STRING("dir", 1);
-		case S_IFREG: RETURN_STRING("file", 1);
+		case S_IFDIR: RETURN_STRING("dir");
+		case S_IFREG: RETURN_STRING("file");
 		}
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Unknown file type (%u)", stat_sb->st_mode & S_IFMT);
-		RETURN_STRING("unknown", 1);
+		php_error_docref(NULL, E_NOTICE, "Unknown file type (%u)", stat_sb->st_mode & S_IFMT);
+		RETURN_STRING("unknown");
 	case FS_IS_W:
 		RETURN_BOOL((stat_sb->st_mode & wmask) != 0);
 	case FS_IS_R:
@@ -539,71 +536,71 @@ static void phar_fancy_stat(struct stat *stat_sb, int type, zval *return_value T
 	case FS_STAT:
 		array_init(return_value);
 
-		MAKE_LONG_ZVAL_INCREF(stat_dev, stat_sb->st_dev);
-		MAKE_LONG_ZVAL_INCREF(stat_ino, stat_sb->st_ino);
-		MAKE_LONG_ZVAL_INCREF(stat_mode, stat_sb->st_mode);
-		MAKE_LONG_ZVAL_INCREF(stat_nlink, stat_sb->st_nlink);
-		MAKE_LONG_ZVAL_INCREF(stat_uid, stat_sb->st_uid);
-		MAKE_LONG_ZVAL_INCREF(stat_gid, stat_sb->st_gid);
+		ZVAL_LONG(&stat_dev, stat_sb->st_dev);
+		ZVAL_LONG(&stat_ino, stat_sb->st_ino);
+		ZVAL_LONG(&stat_mode, stat_sb->st_mode);
+		ZVAL_LONG(&stat_nlink, stat_sb->st_nlink);
+		ZVAL_LONG(&stat_uid, stat_sb->st_uid);
+		ZVAL_LONG(&stat_gid, stat_sb->st_gid);
 #ifdef HAVE_ST_RDEV
-		MAKE_LONG_ZVAL_INCREF(stat_rdev, stat_sb->st_rdev);
+		ZVAL_LONG(&stat_rdev, stat_sb->st_rdev);
 #else
-		MAKE_LONG_ZVAL_INCREF(stat_rdev, -1);
+		ZVAL_LONG(&stat_rdev, -1);
 #endif
-		MAKE_LONG_ZVAL_INCREF(stat_size, stat_sb->st_size);
+		ZVAL_LONG(&stat_size, stat_sb->st_size);
 #ifdef NETWARE
-		MAKE_LONG_ZVAL_INCREF(stat_atime, (stat_sb->st_atime).tv_sec);
-		MAKE_LONG_ZVAL_INCREF(stat_mtime, (stat_sb->st_mtime).tv_sec);
-		MAKE_LONG_ZVAL_INCREF(stat_ctime, (stat_sb->st_ctime).tv_sec);
+		ZVAL_LONG(&stat_atime, (stat_sb->st_atime).tv_sec);
+		ZVAL_LONG(&stat_mtime, (stat_sb->st_mtime).tv_sec);
+		ZVAL_LONG(&stat_ctime, (stat_sb->st_ctime).tv_sec);
 #else
-		MAKE_LONG_ZVAL_INCREF(stat_atime, stat_sb->st_atime);
-		MAKE_LONG_ZVAL_INCREF(stat_mtime, stat_sb->st_mtime);
-		MAKE_LONG_ZVAL_INCREF(stat_ctime, stat_sb->st_ctime);
+		ZVAL_LONG(&stat_atime, stat_sb->st_atime);
+		ZVAL_LONG(&stat_mtime, stat_sb->st_mtime);
+		ZVAL_LONG(&stat_ctime, stat_sb->st_ctime);
 #endif
 #ifdef HAVE_ST_BLKSIZE
-		MAKE_LONG_ZVAL_INCREF(stat_blksize, stat_sb->st_blksize);
+		ZVAL_LONG(&stat_blksize, stat_sb->st_blksize);
 #else
-		MAKE_LONG_ZVAL_INCREF(stat_blksize,-1);
+		ZVAL_LONG(&stat_blksize,-1);
 #endif
 #ifdef HAVE_ST_BLOCKS
-		MAKE_LONG_ZVAL_INCREF(stat_blocks, stat_sb->st_blocks);
+		ZVAL_LONG(&stat_blocks, stat_sb->st_blocks);
 #else
-		MAKE_LONG_ZVAL_INCREF(stat_blocks,-1);
+		ZVAL_LONG(&stat_blocks,-1);
 #endif
 		/* Store numeric indexes in proper order */
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_dev, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_ino, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_mode, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_nlink, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_uid, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_gid, sizeof(zval *), NULL);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_dev);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_ino);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_mode);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_nlink);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_uid);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_gid);
 
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_rdev, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_size, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_atime, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_mtime, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_ctime, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_blksize, sizeof(zval *), NULL);
-		zend_hash_next_index_insert(HASH_OF(return_value), (void *)&stat_blocks, sizeof(zval *), NULL);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_rdev);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_size);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_atime);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_mtime);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_ctime);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_blksize);
+		zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &stat_blocks);
 
 		/* Store string indexes referencing the same zval*/
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[0], strlen(stat_sb_names[0])+1, (void *) &stat_dev, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[1], strlen(stat_sb_names[1])+1, (void *) &stat_ino, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[2], strlen(stat_sb_names[2])+1, (void *) &stat_mode, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[3], strlen(stat_sb_names[3])+1, (void *) &stat_nlink, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[4], strlen(stat_sb_names[4])+1, (void *) &stat_uid, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[5], strlen(stat_sb_names[5])+1, (void *) &stat_gid, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[6], strlen(stat_sb_names[6])+1, (void *) &stat_rdev, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[7], strlen(stat_sb_names[7])+1, (void *) &stat_size, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[8], strlen(stat_sb_names[8])+1, (void *) &stat_atime, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[9], strlen(stat_sb_names[9])+1, (void *) &stat_mtime, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[10], strlen(stat_sb_names[10])+1, (void *) &stat_ctime, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[11], strlen(stat_sb_names[11])+1, (void *) &stat_blksize, sizeof(zval *), NULL);
-		zend_hash_update(HASH_OF(return_value), stat_sb_names[12], strlen(stat_sb_names[12])+1, (void *) &stat_blocks, sizeof(zval *), NULL);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[0], strlen(stat_sb_names[0]), &stat_dev);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[1], strlen(stat_sb_names[1]), &stat_ino);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[2], strlen(stat_sb_names[2]), &stat_mode);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[3], strlen(stat_sb_names[3]), &stat_nlink);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[4], strlen(stat_sb_names[4]), &stat_uid);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[5], strlen(stat_sb_names[5]), &stat_gid);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[6], strlen(stat_sb_names[6]), &stat_rdev);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[7], strlen(stat_sb_names[7]), &stat_size);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[8], strlen(stat_sb_names[8]), &stat_atime);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[9], strlen(stat_sb_names[9]), &stat_mtime);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[10], strlen(stat_sb_names[10]), &stat_ctime);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[11], strlen(stat_sb_names[11]), &stat_blksize);
+		zend_hash_str_update(Z_ARRVAL_P(return_value), stat_sb_names[12], strlen(stat_sb_names[12]), &stat_blocks);
 
 		return;
 	}
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Didn't understand stat call");
+	php_error_docref(NULL, E_WARNING, "Didn't understand stat call");
 	RETURN_FALSE;
 }
 /* }}} */
@@ -617,11 +614,11 @@ static void phar_file_stat(const char *filename, php_stat_len filename_length, i
 	if (!IS_ABSOLUTE_PATH(filename, filename_length) && !strstr(filename, "://")) {
 		char *arch, *entry, *fname;
 		int arch_len, entry_len, fname_len;
-		struct stat sb = {0};
+		zend_stat_t sb = {0};
 		phar_entry_info *data = NULL;
 		phar_archive_data *phar;
 
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		/* we are checking for existence of a file within the relative path.  Chances are good that this is
 		   retrieving something from within the phar archive */
@@ -639,31 +636,31 @@ static void phar_file_stat(const char *filename, php_stat_len filename_length, i
 			phar = PHAR_G(last_phar);
 			goto splitted;
 		}
-		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 
 			efree(entry);
 			entry = estrndup(filename, filename_length);
 			/* fopen within phar, if :// is not in the url, then prepend phar://<archive>/ */
 			entry_len = (int) filename_length;
-			if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL TSRMLS_CC)) {
+			if (FAILURE == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL)) {
 				efree(arch);
 				efree(entry);
 				goto skip_phar;
 			}
 splitted:
-			entry = phar_fix_filepath(entry, &entry_len, 1 TSRMLS_CC);
+			entry = phar_fix_filepath(entry, &entry_len, 1);
 			if (entry[0] == '/') {
-				if (SUCCESS == zend_hash_find(&(phar->manifest), entry + 1, entry_len - 1, (void **) &data)) {
+				if (NULL != (data = zend_hash_str_find_ptr(&(phar->manifest), entry + 1, entry_len - 1))) {
 					efree(entry);
 					goto stat_entry;
 				}
 				goto notfound;
 			}
-			if (SUCCESS == zend_hash_find(&(phar->manifest), entry, entry_len, (void **) &data)) {
+			if (NULL != (data = zend_hash_str_find_ptr(&(phar->manifest), entry, entry_len))) {
 				efree(entry);
 				goto stat_entry;
 			}
-			if (zend_hash_exists(&(phar->virtual_dirs), entry, entry_len)) {
+			if (zend_hash_str_exists(&(phar->virtual_dirs), entry, entry_len)) {
 				efree(entry);
 				efree(arch);
 				if (IS_EXISTS_CHECK(type)) {
@@ -696,8 +693,8 @@ notfound:
 				PHAR_G(cwd) = "/";
 				PHAR_G(cwd_len) = 0;
 				/* clean path without cwd */
-				entry = phar_fix_filepath(entry, &entry_len, 1 TSRMLS_CC);
-				if (SUCCESS == zend_hash_find(&(phar->manifest), entry + 1, entry_len - 1, (void **) &data)) {
+				entry = phar_fix_filepath(entry, &entry_len, 1);
+				if (NULL != (data = zend_hash_str_find_ptr(&(phar->manifest), entry + 1, entry_len - 1))) {
 					PHAR_G(cwd) = save;
 					PHAR_G(cwd_len) = save_len;
 					efree(entry);
@@ -707,7 +704,7 @@ notfound:
 					}
 					goto stat_entry;
 				}
-				if (zend_hash_exists(&(phar->virtual_dirs), entry + 1, entry_len - 1)) {
+				if (zend_hash_str_exists(&(phar->virtual_dirs), entry + 1, entry_len - 1)) {
 					PHAR_G(cwd) = save;
 					PHAR_G(cwd_len) = save_len;
 					efree(entry);
@@ -735,7 +732,7 @@ notfound:
 				efree(arch);
 				/* Error Occurred */
 				if (!IS_EXISTS_CHECK(type)) {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING, "%sstat failed for %s", IS_LINK_OPERATION(type) ? "L" : "", filename);
+					php_error_docref(NULL, E_WARNING, "%sstat failed for %s", IS_LINK_OPERATION(type) ? "L" : "", filename);
 				}
 				RETURN_FALSE;
 			}
@@ -795,7 +792,7 @@ statme_baby:
 			sb.st_blksize = -1;
 			sb.st_blocks = -1;
 #endif
-			phar_fancy_stat(&sb, type, return_value TSRMLS_CC);
+			phar_fancy_stat(&sb, type, return_value);
 			return;
 		}
 	}
@@ -811,9 +808,9 @@ void fname(INTERNAL_FUNCTION_PARAMETERS) { \
 		PHAR_G(orig)(INTERNAL_FUNCTION_PARAM_PASSTHRU); \
 	} else { \
 		char *filename; \
-		int filename_len; \
+		size_t filename_len; \
 		\
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "p", &filename, &filename_len) == FAILURE) { \
+		if (zend_parse_parameters(ZEND_NUM_ARGS(), "p", &filename, &filename_len) == FAILURE) { \
 			return; \
 		} \
 		\
@@ -895,23 +892,23 @@ PharFileFunction(phar_is_dir, FS_IS_DIR, orig_is_dir)
 PHAR_FUNC(phar_is_file) /* {{{ */
 {
 	char *filename;
-	int filename_len;
+	size_t filename_len;
 
 	if (!PHAR_G(intercepted)) {
 		goto skip_phar;
 	}
 
-	if ((PHAR_GLOBALS->phar_fname_map.arBuckets && !zend_hash_num_elements(&(PHAR_GLOBALS->phar_fname_map)))
-		&& !cached_phars.arBuckets) {
+	if ((PHAR_G(phar_fname_map.u.flags) && !zend_hash_num_elements(&(PHAR_G(phar_fname_map))))
+		&& !cached_phars.u.flags) {
 		goto skip_phar;
 	}
-	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "p", &filename, &filename_len) == FAILURE) {
+	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS(), "p", &filename, &filename_len) == FAILURE) {
 		goto skip_phar;
 	}
 	if (!IS_ABSOLUTE_PATH(filename, filename_len) && !strstr(filename, "://")) {
 		char *arch, *entry, *fname;
 		int arch_len, entry_len, fname_len;
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		/* we are checking for existence of a file within the relative path.  Chances are good that this is
 		   retrieving something from within the phar archive */
@@ -920,7 +917,7 @@ PHAR_FUNC(phar_is_file) /* {{{ */
 			goto skip_phar;
 		}
 		fname_len = strlen(fname);
-		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 			phar_archive_data *phar;
 
 			efree(entry);
@@ -928,12 +925,12 @@ PHAR_FUNC(phar_is_file) /* {{{ */
 			/* fopen within phar, if :// is not in the url, then prepend phar://<archive>/ */
 			entry_len = filename_len;
 			/* retrieving a file within the current directory, so use this if possible */
-			if (SUCCESS == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL TSRMLS_CC)) {
+			if (SUCCESS == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL)) {
 				phar_entry_info *etemp;
 
-				entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1 TSRMLS_CC);
+				entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1);
 				if (entry[0] == '/') {
-					if (SUCCESS == zend_hash_find(&(phar->manifest), entry + 1, entry_len - 1, (void **) &etemp)) {
+					if (NULL != (etemp = zend_hash_str_find_ptr(&(phar->manifest), entry + 1, entry_len - 1))) {
 						/* this file is not in the current directory, use the original path */
 found_it:
 						efree(entry);
@@ -941,7 +938,7 @@ found_it:
 						RETURN_BOOL(!etemp->is_dir);
 					}
 				} else {
-					if (SUCCESS == zend_hash_find(&(phar->manifest), entry, entry_len, (void **) &etemp)) {
+					if (NULL != (etemp = zend_hash_str_find_ptr(&(phar->manifest), entry, entry_len))) {
 						goto found_it;
 					}
 				}
@@ -962,23 +959,23 @@ skip_phar:
 PHAR_FUNC(phar_is_link) /* {{{ */
 {
 	char *filename;
-	int filename_len;
+	size_t filename_len;
 
 	if (!PHAR_G(intercepted)) {
 		goto skip_phar;
 	}
 
-	if ((PHAR_GLOBALS->phar_fname_map.arBuckets && !zend_hash_num_elements(&(PHAR_GLOBALS->phar_fname_map)))
-		&& !cached_phars.arBuckets) {
+	if ((PHAR_G(phar_fname_map.u.flags) && !zend_hash_num_elements(&(PHAR_G(phar_fname_map))))
+		&& !cached_phars.u.flags) {
 		goto skip_phar;
 	}
-	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "p", &filename, &filename_len) == FAILURE) {
+	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS(), "p", &filename, &filename_len) == FAILURE) {
 		goto skip_phar;
 	}
 	if (!IS_ABSOLUTE_PATH(filename, filename_len) && !strstr(filename, "://")) {
 		char *arch, *entry, *fname;
 		int arch_len, entry_len, fname_len;
-		fname = (char*)zend_get_executed_filename(TSRMLS_C);
+		fname = (char*)zend_get_executed_filename();
 
 		/* we are checking for existence of a file within the relative path.  Chances are good that this is
 		   retrieving something from within the phar archive */
@@ -987,7 +984,7 @@ PHAR_FUNC(phar_is_link) /* {{{ */
 			goto skip_phar;
 		}
 		fname_len = strlen(fname);
-		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0 TSRMLS_CC)) {
+		if (SUCCESS == phar_split_fname(fname, fname_len, &arch, &arch_len, &entry, &entry_len, 2, 0)) {
 			phar_archive_data *phar;
 
 			efree(entry);
@@ -995,12 +992,12 @@ PHAR_FUNC(phar_is_link) /* {{{ */
 			/* fopen within phar, if :// is not in the url, then prepend phar://<archive>/ */
 			entry_len = filename_len;
 			/* retrieving a file within the current directory, so use this if possible */
-			if (SUCCESS == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL TSRMLS_CC)) {
+			if (SUCCESS == phar_get_archive(&phar, arch, arch_len, NULL, 0, NULL)) {
 				phar_entry_info *etemp;
 
-				entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1 TSRMLS_CC);
+				entry = phar_fix_filepath(estrndup(entry, entry_len), &entry_len, 1);
 				if (entry[0] == '/') {
-					if (SUCCESS == zend_hash_find(&(phar->manifest), entry + 1, entry_len - 1, (void **) &etemp)) {
+					if (NULL != (etemp = zend_hash_str_find_ptr(&(phar->manifest), entry + 1, entry_len - 1))) {
 						/* this file is not in the current directory, use the original path */
 found_it:
 						efree(entry);
@@ -1008,7 +1005,7 @@ found_it:
 						RETURN_BOOL(etemp->link);
 					}
 				} else {
-					if (SUCCESS == zend_hash_find(&(phar->manifest), entry, entry_len, (void **) &etemp)) {
+					if (NULL != (etemp = zend_hash_str_find_ptr(&(phar->manifest), entry, entry_len))) {
 						goto found_it;
 					}
 				}
@@ -1034,8 +1031,8 @@ PharFileFunction(phar_lstat, FS_LSTAT, orig_lstat)
 PharFileFunction(phar_stat, FS_STAT, orig_stat)
 /* }}} */
 
-/* {{{ void phar_intercept_functions(TSRMLS_D) */
-void phar_intercept_functions(TSRMLS_D)
+/* {{{ void phar_intercept_functions(void) */
+void phar_intercept_functions(void)
 {
 	if (!PHAR_G(request_init)) {
 		PHAR_G(cwd) = NULL;
@@ -1045,22 +1042,22 @@ void phar_intercept_functions(TSRMLS_D)
 }
 /* }}} */
 
-/* {{{ void phar_release_functions(TSRMLS_D) */
-void phar_release_functions(TSRMLS_D)
+/* {{{ void phar_release_functions(void) */
+void phar_release_functions(void)
 {
 	PHAR_G(intercepted) = 0;
 }
 /* }}} */
 
-/* {{{ void phar_intercept_functions_init(TSRMLS_D) */
+/* {{{ void phar_intercept_functions_init(void) */
 #define PHAR_INTERCEPT(func) \
 	PHAR_G(orig_##func) = NULL; \
-	if (SUCCESS == zend_hash_find(CG(function_table), #func, sizeof(#func), (void **)&orig)) { \
+	if (NULL != (orig = zend_hash_str_find_ptr(CG(function_table), #func, sizeof(#func)-1))) { \
 		PHAR_G(orig_##func) = orig->internal_function.handler; \
 		orig->internal_function.handler = phar_##func; \
 	}
 
-void phar_intercept_functions_init(TSRMLS_D)
+void phar_intercept_functions_init(void)
 {
 	zend_function *orig;
 
@@ -1090,14 +1087,14 @@ void phar_intercept_functions_init(TSRMLS_D)
 }
 /* }}} */
 
-/* {{{ void phar_intercept_functions_shutdown(TSRMLS_D) */
+/* {{{ void phar_intercept_functions_shutdown(void) */
 #define PHAR_RELEASE(func) \
-	if (PHAR_G(orig_##func) && SUCCESS == zend_hash_find(CG(function_table), #func, sizeof(#func), (void **)&orig)) { \
+	if (PHAR_G(orig_##func) && NULL != (orig = zend_hash_str_find_ptr(CG(function_table), #func, sizeof(#func)-1))) { \
 		orig->internal_function.handler = PHAR_G(orig_##func); \
 	} \
 	PHAR_G(orig_##func) = NULL;
 
-void phar_intercept_functions_shutdown(TSRMLS_D)
+void phar_intercept_functions_shutdown(void)
 {
 	zend_function *orig;
 
@@ -1151,7 +1148,7 @@ static struct _phar_orig_functions {
 	void        (*orig_stat)(INTERNAL_FUNCTION_PARAMETERS);
 } phar_orig_functions = {NULL};
 
-void phar_save_orig_functions(TSRMLS_D) /* {{{ */
+void phar_save_orig_functions(void) /* {{{ */
 {
 	phar_orig_functions.orig_fopen             = PHAR_G(orig_fopen);
 	phar_orig_functions.orig_file_get_contents = PHAR_G(orig_file_get_contents);
@@ -1178,7 +1175,7 @@ void phar_save_orig_functions(TSRMLS_D) /* {{{ */
 }
 /* }}} */
 
-void phar_restore_orig_functions(TSRMLS_D) /* {{{ */
+void phar_restore_orig_functions(void) /* {{{ */
 {
 	PHAR_G(orig_fopen)             = phar_orig_functions.orig_fopen;
 	PHAR_G(orig_file_get_contents) = phar_orig_functions.orig_file_get_contents;
@@ -1213,4 +1210,3 @@ void phar_restore_orig_functions(TSRMLS_D) /* {{{ */
  * vim600: noet sw=4 ts=4 fdm=marker
  * vim<600: noet sw=4 ts=4
  */
-
